// dig

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

`include "config.v" // RR20150511 common config options

`include "cog.v"    // cog logic and memory (8 instances)
`include "hub.v"    // hub logic and memory

module              dig
(
input               nres,           // reset input (active low)

output       [7:0]  cfg,            // configuration output (set by clkset instruction)

input               clk_cog,        // cog clock input
input               clk_pll,        // pll simulator clock input (2x cog clock)

input       [31:0]  pin_in,         // pin state inputs
output      [31:0]  pin_out,        // pin state outputs
output      [31:0]  pin_dir,        // pin direction outputs

output      [7:0]  cog_led          // led outputs to show which cogs are active
);


// cnt

reg [31:0] cnt;

always @(posedge clk_cog)
if (nres)
    cnt <= cnt + 1'b1;


// bus enable

reg ena_bus;

always @(posedge clk_cog or negedge nres)
if (!nres)
    ena_bus <= 1'b0;
else
`ifdef HUB_SINGLE_CLOCK
    ena_bus <= clk_cog;                         // hub 1:8
`else
    ena_bus <= !ena_bus;                        // hub 1:16
`endif	


// bus select

reg [NCOGS-1:0] bus_sel;

always @(posedge clk_cog or negedge nres)
if (!nres)
    bus_sel <= 1'b0;
else if (ena_bus)
    bus_sel <= {bus_sel[NCOGS-2:0], ~|bus_sel[NCOGS-2:0]};	// rotate hub slot


// cogs

wire [NCOGS-1:0]          bus_r;
wire [NCOGS-1:0]          bus_e;
wire [NCOGS-1:0]          bus_w;
wire [NCOGS-1:0]  [1:0]   bus_s;
wire [NCOGS-1:0] [15:0]   bus_a;
wire [NCOGS-1:0] [31:0]   bus_d;
wire [NCOGS-1:0]          pll;
wire [NCOGS-1:0] [31:0]   outx;
wire [NCOGS-1:0] [31:0]   dirx;

genvar COG_ID;
generate
    for (COG_ID=0; COG_ID<NCOGS; COG_ID=COG_ID+1)		// instantiate "n" cogs
    begin : coggen
        cog #(COG_ID) cog_(
                    .nres       (nres),
                    .clk_cog    (clk_cog),
                    .clk_pll    (clk_pll),
                    .ena_bus    (ena_bus),
                    .ptr_w      (ptr_w[COG_ID]),
                    .ptr_d      (ptr_d),
                    .ena        (cog_ena[COG_ID]),
                    .bus_sel    (bus_sel[COG_ID]),
                    .bus_r      (bus_r[COG_ID]),
                    .bus_e      (bus_e[COG_ID]),
                    .bus_w      (bus_w[COG_ID]),
                    .bus_s      (bus_s[COG_ID]),
                    .bus_a      (bus_a[COG_ID]),
                    .bus_d      (bus_d[COG_ID]),
                    .bus_q      (bus_q),
                    .bus_c      (bus_c),
                    .bus_ack    (bus_ack[COG_ID]),
                    .cnt        (cnt),
                    .pll_in     (pll),
                    .pll_out    (pll[COG_ID]),
                    .pin_in     (pin_in),
                    .pin_out    (outx[COG_ID]),
                    .pin_dir    (dirx[COG_ID])   );
    end
endgenerate


// hub

wire        hub_bus_r   = |bus_r;
wire        hub_bus_e   = |bus_e;
wire        hub_bus_w   = |bus_w;

`ifdef COGS_8
wire  [1:0] hub_bus_s   = bus_s[7] | bus_s[6] | bus_s[5] | bus_s[4] | bus_s[3] | bus_s[2] | bus_s[1] | bus_s[0];
wire [15:0] hub_bus_a   = bus_a[7] | bus_a[6] | bus_a[5] | bus_a[4] | bus_a[3] | bus_a[2] | bus_a[1] | bus_a[0];
wire [31:0] hub_bus_d   = bus_d[7] | bus_d[6] | bus_d[5] | bus_d[4] | bus_d[3] | bus_d[2] | bus_d[1] | bus_d[0];
`elsif COGS_4
wire  [1:0] hub_bus_s   = bus_s[3] | bus_s[2] | bus_s[1] | bus_s[0];
wire [15:0] hub_bus_a   = bus_a[3] | bus_a[2] | bus_a[1] | bus_a[0];
wire [31:0] hub_bus_d   = bus_d[3] | bus_d[2] | bus_d[1] | bus_d[0];
`endif


wire  [31:0]      bus_q;
wire              bus_c;
wire  [NCOGS-1:0] bus_ack;
wire  [NCOGS-1:0] cog_ena;
wire  [NCOGS-1:0] ptr_w;
wire  [27:0]      ptr_d;

hub hub_        (   .clk_cog    (clk_cog),
                    .ena_bus    (ena_bus),
                    .nres       (nres),
                    .bus_sel    (bus_sel),
                    .bus_r      (hub_bus_r),
                    .bus_e      (hub_bus_e),
                    .bus_w      (hub_bus_w),
                    .bus_s      (hub_bus_s),
                    .bus_a      (hub_bus_a),
                    .bus_d      (hub_bus_d),
                    .bus_q      (bus_q),
                    .bus_c      (bus_c),
                    .bus_ack    (bus_ack),
                    .cog_ena    (cog_ena),
                    .ptr_w      (ptr_w),
                    .ptr_d      (ptr_d),
                    .cfg        (cfg)   );


// pins

`ifdef COGS_8
assign pin_out      = outx[7] | outx[6] | outx[5] | outx[4] | outx[3] | outx[2] | outx[1] | outx[0];
assign pin_dir      = dirx[7] | dirx[6] | dirx[5] | dirx[4] | dirx[3] | dirx[2] | dirx[1] | dirx[0];
`elsif COGS_4
assign pin_out      = outx[3] | outx[2] | outx[1] | outx[0];
assign pin_dir      = dirx[3] | dirx[2] | dirx[1] | dirx[0];
`endif

// cog leds

`ifdef INVERT_COG_LEDS
assign cog_led      = !cog_ena;
`else
assign cog_led      = cog_ena;
`endif

endmodule
